@@ -4,27 +4,30 @@
 #
 # This software may be used and distributed according to the terms
 # of the GNU General Public License, incorporated herein by reference.
+'''converting foreign VCS repositories to Mercurial'''
 
 import convcmd
 from mercurial import commands
+from mercurial.i18n import _
 
 # Commands definition was moved elsewhere to ease demandload job.
 
 def convert(ui, src, dest=None, revmapfile=None, **opts):
     """Convert a foreign SCM repository to a Mercurial one.
 
-    Accepted source formats:
-    - Mercurial
-    - CVS
-    - Darcs
-    - git
-    - Subversion
-    - Monotone
-    - GNU Arch
+    Accepted source formats [identifiers]:
+    - Mercurial [hg]
+    - CVS [cvs]
+    - Darcs [darcs]
+    - git [git]
+    - Subversion [svn]
+    - Monotone [mtn]
+    - GNU Arch [gnuarch]
+    - Bazaar [bzr]
 
-    Accepted destination formats:
-    - Mercurial
-    - Subversion (history on branches is not preserved)
+    Accepted destination formats [identifiers]:
+    - Mercurial [hg]
+    - Subversion [svn] (history on branches is not preserved)
 
     If no revision is given, all revisions will be converted. Otherwise,
     convert will only import up to the named revision (given in a format
@@ -82,8 +85,59 @@ def convert(ui, src, dest=None, revmapfile=None, **opts):
     Mercurial Source
     -----------------
 
+    --config convert.hg.ignoreerrors=False (boolean)
+        ignore integrity errors when reading. Use it to fix Mercurial
+        repositories with missing revlogs, by converting from and to
+        Mercurial.
     --config convert.hg.saverev=True          (boolean)
         allow target to preserve source revision ID
+    --config convert.hg.startrev=0            (hg revision identifier)
+        convert start revision and its descendants
+
+    CVS Source
+    ----------
+
+    CVS source will use a sandbox (i.e. a checked-out copy) from CVS
+    to indicate the starting point of what will be converted. Direct
+    access to the repository files is not needed, unless of course
+    the repository is :local:. The conversion uses the top level
+    directory in the sandbox to find the CVS repository, and then uses
+    CVS rlog commands to find files to convert. This means that unless
+    a filemap is given, all files under the starting directory will be
+    converted, and that any directory reorganisation in the CVS
+    sandbox is ignored.
+
+    Because CVS does not have changesets, it is necessary to collect
+    individual commits to CVS and merge them into changesets. CVS
+    source uses its internal changeset merging code by default but can
+    be configured to call the external 'cvsps' program by setting:
+        --config convert.cvsps='cvsps -A -u --cvs-direct -q'
+    This is a legacy option and may be removed in future.
+
+    The options shown are the defaults.
+
+    Internal cvsps is selected by setting
+        --config convert.cvsps=builtin
+    and has a few more configurable options:
+        --config convert.cvsps.fuzz=60   (integer)
+            Specify the maximum time (in seconds) that is allowed between
+            commits with identical user and log message in a single
+            changeset. When very large files were checked in as part
+            of a changeset then the default may not be long enough.
+        --config convert.cvsps.mergeto='{{mergetobranch ([-\w]+)}}'
+            Specify a regular expression to which commit log messages are
+            matched. If a match occurs, then the conversion process will
+            insert a dummy revision merging the branch on which this log
+            message occurs to the branch indicated in the regex.
+        --config convert.cvsps.mergefrom='{{mergefrombranch ([-\w]+)}}'
+            Specify a regular expression to which commit log messages are
+            matched. If a match occurs, then the conversion process will
+            add the most recent revision on the branch indicated in the
+            regex as the second parent of the changeset.
+
+    The hgext/convert/cvsps wrapper script allows the builtin changeset
+    merging code to be run without doing a conversion. Its parameters and
+    output are similar to that of cvsps 2.1.
 
     Subversion Source
     -----------------
@@ -134,14 +188,14 @@ commands.norepo += " convert debugsvnlog"
 cmdtable = {
     "convert":
         (convert,
-         [('A', 'authors', '', 'username mapping filename'),
-          ('d', 'dest-type', '', 'destination repository type'),
-          ('', 'filemap', '', 'remap file names using contents of file'),
-          ('r', 'rev', '', 'import up to target revision REV'),
-          ('s', 'source-type', '', 'source repository type'),
-          ('', 'splicemap', '', 'splice synthesized history into place'),
-          ('', 'datesort', None, 'try to sort changesets by date')],
-         'hg convert [OPTION]... SOURCE [DEST [REVMAP]]'),
+         [('A', 'authors', '', _('username mapping filename')),
+          ('d', 'dest-type', '', _('destination repository type')),
+          ('', 'filemap', '', _('remap file names using contents of file')),
+          ('r', 'rev', '', _('import up to target revision REV')),
+          ('s', 'source-type', '', _('source repository type')),
+          ('', 'splicemap', '', _('splice synthesized history into place')),
+          ('', 'datesort', None, _('try to sort changesets by date'))],
+         _('hg convert [OPTION]... SOURCE [DEST [REVMAP]]')),
     "debugsvnlog":
         (debugsvnlog,
          [],
