@@ -2,7 +2,7 @@
 import base64, errno
 import os
 import cPickle as pickle
-from mercurial import util
+from mercurial import util, strutil
 from mercurial.i18n import _
 
 def encodeargs(args):
@@ -68,17 +68,23 @@ class converter_source(object):
         raise NotImplementedError()
 
     def getfile(self, name, rev):
-        """Return file contents as a string"""
+        """Return file contents as a string. rev is the identifier returned
+        by a previous call to getchanges().
+        """
         raise NotImplementedError()
 
     def getmode(self, name, rev):
-        """Return file mode, eg. '', 'x', or 'l'"""
+        """Return file mode, eg. '', 'x', or 'l'. rev is the identifier
+        returned by a previous call to getchanges().
+        """
         raise NotImplementedError()
 
     def getchanges(self, version):
-        """Returns a tuple of (files, copies)
-        Files is a sorted list of (filename, id) tuples for all files changed
-        in version, where id is the source revision id of the file.
+        """Returns a tuple of (files, copies).
+
+        files is a sorted list of (filename, id) tuples for all files
+        changed between version and it's first parent returned by
+        getcommit(). id is the source revision id of the file.
 
         copies is a dictionary of dest: source
         """
@@ -153,26 +159,18 @@ class converter_sink(object):
         mapping equivalent authors identifiers for each system."""
         return None
 
-    def putfile(self, f, e, data):
-        """Put file for next putcommit().
-        f: path to file
-        e: '', 'x', or 'l' (regular file, executable, or symlink)
-        data: file contents"""
-        raise NotImplementedError()
-
-    def delfile(self, f):
-        """Delete file for next putcommit().
-        f: path to file"""
-        raise NotImplementedError()
-
-    def putcommit(self, files, parents, commit):
+    def putcommit(self, files, copies, parents, commit, source):
         """Create a revision with all changed files listed in 'files'
         and having listed parents. 'commit' is a commit object containing
         at a minimum the author, date, and message for this changeset.
-        Called after putfile() and delfile() calls. Note that the sink
-        repository is not told to update itself to a particular revision
-        (or even what that revision would be) before it receives the
-        file data."""
+        'files' is a list of (path, version) tuples, 'copies'is a dictionary
+        mapping destinations to sources, and 'source' is the source repository.
+        Only getfile() and getmode() should be called on 'source'.
+
+        Note that the sink repository is not told to update itself to
+        a particular revision (or even what that revision would be)
+        before it receives the file data.
+        """
         raise NotImplementedError()
 
     def puttags(self, tags):
@@ -181,7 +179,7 @@ class converter_sink(object):
         raise NotImplementedError()
 
     def setbranch(self, branch, pbranches):
-        """Set the current branch name. Called before the first putfile
+        """Set the current branch name. Called before the first putcommit
         on the branch.
         branch: branch name for subsequent commits
         pbranches: (converted parent revision, parent branch) tuples"""
@@ -232,11 +230,11 @@ class commandline(object):
         cmdline = [util.shellquote(arg) for arg in cmdline]
         cmdline += ['2>', util.nulldev, '<', util.nulldev]
         cmdline = ' '.join(cmdline)
-        self.ui.debug(cmdline, '\n')
         return cmdline
 
     def _run(self, cmd, *args, **kwargs):
         cmdline = self._cmdline(cmd, *args, **kwargs)
+        self.ui.debug(_('running: %s\n') % (cmdline,))
         self.prerun()
         try:
             return util.popen(cmdline)
@@ -334,7 +332,7 @@ class mapfile(dict):
                 raise
             return
         for line in fp:
-            key, value = line[:-1].split(' ', 1)
+            key, value = strutil.rsplit(line[:-1], ' ', 1)
             if key not in self:
                 self.order.append(key)
             super(mapfile, self).__setitem__(key, value)
