@@ -16,6 +16,7 @@ import win32api
 import errno, os, sys, pywintypes, win32con, win32file, win32process
 import cStringIO, winerror
 import osutil
+import util
 from win32com.shell import shell,shellcon
 
 class WinError:
@@ -201,21 +202,17 @@ def lookup_reg(key, valname=None, scope=None):
     except ImportError:
         return None
 
-    def query_val(scope, key, valname):
-        try:
-            keyhandle = OpenKey(scope, key)
-            return QueryValueEx(keyhandle, valname)[0]
-        except EnvironmentError:
-            return None
-
     if scope is None:
         scope = (HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE)
     elif not isinstance(scope, (list, tuple)):
         scope = (scope,)
     for s in scope:
-        val = query_val(s, key, valname)
-        if val is not None:
-            return val
+        try:
+            val = QueryValueEx(OpenKey(s, key), valname)[0]
+            # never let a Unicode string escape into the wild
+            return util.tolocal(val.encode('UTF-8'))
+        except EnvironmentError:
+            pass
 
 def system_rcpath_win32():
     '''return default os-specific hgrc search path'''
@@ -248,7 +245,7 @@ def system_rcpath_win32():
 def user_rcpath_win32():
     '''return os-specific hgrc search path to the user dir'''
     userdir = os.path.expanduser('~')
-    if sys.getwindowsversion() != 2 and userdir == '~':
+    if sys.getwindowsversion()[3] != 2 and userdir == '~':
         # We are on win < nt: fetch the APPDATA directory location and use
         # the parent directory as the user home dir.
         appdir = shell.SHGetPathFromIDList(
@@ -295,7 +292,7 @@ class posixfile_nt(object):
             raise WinIOError(err, name)
 
     def __iter__(self):
-        for line in self.read().splitlines(True):
+        for line in self.readlines():
             yield line
 
     def read(self, count=-1):
@@ -314,6 +311,11 @@ class posixfile_nt(object):
         except pywintypes.error, err:
             raise WinIOError(err)
 
+    def readlines(self, sizehint=None):
+        # splitlines() splits on single '\r' while readlines()
+        # does not. cStringIO has a well behaving readlines() and is fast.
+        return cStringIO.StringIO(self.read()).readlines()
+
     def write(self, data):
         try:
             if 'a' in self.mode:
